@@ -14,40 +14,16 @@
 #ifndef FC__OVERVIEW_COMMON_H
 #define FC__OVERVIEW_COMMON_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #include "fc_types.h"
 
 #include "canvas_g.h"
 
 #include "options.h"
 
-enum overview_layers {
-  OLAYER_BACKGROUND,
-  OLAYER_RELIEF,
-  OLAYER_BORDERS,
-  OLAYER_BORDERS_ON_OCEAN,
-  OLAYER_UNITS,
-  OLAYER_CITIES,
-  OLAYER_COUNT
-};
-
-/* Holds all information about the overview aka minimap. */
-struct overview {
-  /* The following fields are controlled by mapview_common.c. */
-  double map_x0, map_y0; /* Origin of the overview, in natural coords. */
-  int width, height;		/* Size in pixels. */
-
-  /* Holds the map, unwrapped. */
-  struct canvas *map;
-
-  /* A backing store for the window itself, wrapped. */
-  struct canvas *window;
-
-  bool fog;
-  bool layers[OLAYER_COUNT];
-};
-
-extern struct overview overview;
-
 /* The overview tile width and height are defined in terms of the base
  * size.  For iso-maps the width is twice the height since "natural"
  * coordinates are used.  For classical maps the width and height are
@@ -62,13 +38,19 @@ void overview_to_map_pos(int *map_x, int *map_y,
 			 int overview_x, int overview_y);
 
 void refresh_overview_canvas(void);
+void refresh_overview_from_canvas(void);
 void overview_update_tile(struct tile *ptile);
 void calculate_overview_dimensions(void);
+void overview_free(void);
 
 void center_tile_overviewcanvas(void);
 
 void flush_dirty_overview(void);
 
-void overview_redraw_callback(struct client_option *option);
+void overview_redraw_callback(struct option *option);
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
 
 #endif /* FC__OVERVIEW_COMMON_H */
