@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 2005 The Freeciv Team
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,11 +11,9 @@
    GNU General Public License for more details.
 ***********************************************************************/
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
-#include <assert.h>
-
 #include <unistd.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -27,7 +25,7 @@
 #include "string_vector.h"
 #include "support.h"
 
-/* include */
+/* client/include */
 #include "themes_g.h"
 
 /* client */
@@ -44,7 +42,7 @@
   
   Theme is stored in a directory called like the theme. The directory contains
   some data files. Each gui defines its own format in the
-  get_useable_themes_in_directory() function.
+  get_usable_themes_in_directory() function.
 ****************************************************************************/
 
 /* A directory containing a list of usable themes */
@@ -61,34 +59,35 @@ struct theme_directory {
 static int num_directories;
 struct theme_directory *directories;
 
-/****************************************************************************
+/************************************************************************//**
   Initialized themes data
 ****************************************************************************/
 void init_themes(void)
 {
   int i;
-    
-  /* get GUI-specific theme directories */
+
+  /* Get GUI-specific theme directories */
   char **gui_directories =
       get_gui_specific_themes_directories(&num_directories);
-  
-  directories = 
+
+  directories =
       fc_malloc(sizeof(struct theme_directory) * num_directories);
-          
+
   for (i = 0; i < num_directories; i++) {
     directories[i].path = gui_directories[i];
-    
-    /* get useable themes in this directory */
+
+    /* Get usable themes in this directory */
     directories[i].themes =
-	get_useable_themes_in_directory(directories[i].path,
-					&(directories[i].num_themes));
+	get_usable_themes_in_directory(directories[i].path,
+                                       &(directories[i].num_themes));
   }
+  free(gui_directories);
 }
 
-/****************************************************************************
-  Return a static string vector of useable theme names.
+/************************************************************************//**
+  Return a static string vector of usable theme names.
 ****************************************************************************/
-const struct strvec *get_themes_list(void)
+const struct strvec *get_themes_list(const struct option *poption)
 {
   static struct strvec *themes_list = NULL;
 
@@ -114,31 +113,32 @@ const struct strvec *get_themes_list(void)
   return themes_list;
 }
 
-/****************************************************************************
+/************************************************************************//**
   Loads a theme with the given name. First matching directory will be used.
   If there's no such theme the function returns FALSE.
 ****************************************************************************/
 bool load_theme(const char *theme_name)
 {
   int i, j;
+
   for (i = 0; i < num_directories; i++) {
     for (j = 0; j < directories[i].num_themes; j++) {
       if (strcmp(theme_name, directories[i].themes[j]) == 0) {
-	gui_load_theme(directories[i].path, directories[i].themes[j]);
-	return TRUE;
+        gui_load_theme(directories[i].path, directories[i].themes[j]);
+        return TRUE;
       }
     }
   }
   return FALSE;
 }
 
-/****************************************************************************
+/************************************************************************//**
   Wrapper for load_theme. It's is used by local options dialog
 ****************************************************************************/
-void theme_reread_callback(struct client_option *poption)
+void theme_reread_callback(struct option *poption)
 {
   const char *theme_name = option_str_get(poption);
 
-  RETURN_IF_FAIL(NULL != theme_name && theme_name[0] != '\0');
+  fc_assert_ret(NULL != theme_name && theme_name[0] != '\0');
   load_theme(theme_name);
 }
