@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,31 +12,31 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdio.h>
 #include <stdlib.h>
 
 /* utility */
+#include "bitvector.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
 
 /* common */
+#include "ai.h"
 #include "diptreaty.h"
 #include "events.h"
 #include "game.h"
 #include "map.h"
 #include "packets.h"
 #include "player.h"
+#include "research.h"
 #include "unit.h"
 
-/* ai */
-#include "advdiplomacy.h"
-
-/* scripting */
-#include "script.h"
+/* common/scriptcore */
+#include "luascript_types.h"
 
 /* server */
 #include "citytools.h"
@@ -44,67 +44,64 @@
 #include "maphand.h"
 #include "plrhand.h"
 #include "notify.h"
-#include "settlers.h"
 #include "techtools.h"
 #include "unittools.h"
 
+/* server/advisors */
+#include "autosettlers.h"
+
+/* server/scripting */
+#include "script_server.h"
+
 #include "diplhand.h"
 
-#define SPECLIST_TAG treaty
-#define SPECLIST_TYPE struct Treaty
-#include "speclist.h"
-
-#define treaty_list_iterate(list, p) \
-    TYPED_LIST_ITERATE(struct Treaty, list, p)
-#define treaty_list_iterate_end  LIST_ITERATE_END
-
 static struct treaty_list *treaties = NULL;
 
 /* FIXME: Should this be put in a ruleset somewhere? */
 #define TURNS_LEFT 16
 
-/**************************************************************************
-  Calls treaty_evaluate function if such is set for AI player                           
+/**********************************************************************//**
+  Calls treaty_evaluate function if such is set for AI player.    
 **************************************************************************/
 static void call_treaty_evaluate(struct player *pplayer, struct player *aplayer,
                                  struct Treaty *ptreaty)
 {
-  if (pplayer->ai_data.control && pplayer->ai->funcs.treaty_evaluate) {
-    pplayer->ai->funcs.treaty_evaluate(pplayer, aplayer, ptreaty);
+  if (is_ai(pplayer)) {
+    CALL_PLR_AI_FUNC(treaty_evaluate, pplayer, pplayer, aplayer, ptreaty);
   }
 }
 
-/**************************************************************************
-  Calls treaty_accepted function if such is set for AI player
+/**********************************************************************//**
+  Calls treaty_accepted function if such is set for AI player.
 **************************************************************************/
 static void call_treaty_accepted(struct player *pplayer, struct player *aplayer,
                                  struct Treaty *ptreaty)
 {
-  if (pplayer->ai_data.control && pplayer->ai->funcs.treaty_accepted) {
-    pplayer->ai->funcs.treaty_accepted(pplayer, aplayer, ptreaty);
+  if (is_ai(pplayer)) {
+    CALL_PLR_AI_FUNC(treaty_accepted, pplayer, pplayer, aplayer, ptreaty);
   }
 }
 
-/**************************************************************************
-...
+/**********************************************************************//**
+  Initialize diplhand module
 **************************************************************************/
 void diplhand_init(void)
 {
   treaties = treaty_list_new();
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Free all the resources allocated by diplhand.
 **************************************************************************/
 void diplhand_free(void)
 {
   free_treaties();
 
-  treaty_list_free(treaties);
+  treaty_list_destroy(treaties);
   treaties = NULL;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Free all the treaties currently in treaty list.
 **************************************************************************/
 void free_treaties(void)
@@ -118,14 +115,14 @@ void free_treaties(void)
   treaty_list_clear(treaties);
 }
 
-/**************************************************************************
-...
+/**********************************************************************//**
+  Find currently active treaty between two players.
 **************************************************************************/
 struct Treaty *find_treaty(struct player *plr0, struct player *plr1)
 {
   treaty_list_iterate(treaties, ptreaty) {
-    if ((ptreaty->plr0 == plr0 && ptreaty->plr1 == plr1) ||
-	(ptreaty->plr0 == plr1 && ptreaty->plr1 == plr0)) {
+    if ((ptreaty->plr0 == plr0 && ptreaty->plr1 == plr1)
+        || (ptreaty->plr0 == plr1 && ptreaty->plr1 == plr0)) {
       return ptreaty;
     }
   } treaty_list_iterate_end;
@@ -133,10 +130,56 @@ struct Treaty *find_treaty(struct player *plr0, struct player *plr1)
   return NULL;
 }
 
-/**************************************************************************
-pplayer clicked the accept button. If he accepted the treaty we check the
-clauses. If both players have now accepted the treaty we execute the agreed
-clauses.
+/**********************************************************************//**
+  Return the closest of the two diplstate types.
+**************************************************************************/
+static enum diplstate_type dst_closest(enum diplstate_type a,
+                                       enum diplstate_type b)
+{
+  static const int how_close[DS_LAST] = {
+    [DS_NO_CONTACT] = 0,
+    [DS_WAR] = 1,
+    [DS_CEASEFIRE] = 2,
+    [DS_ARMISTICE] = 3,
+    [DS_PEACE] = 4,
+    [DS_ALLIANCE] = 5,
+    [DS_TEAM] = 6,
+  };
+
+  if (how_close[a] < how_close[b]) {
+    return b;
+  } else {
+    return a;
+  }
+}
+
+/**********************************************************************//**
+  Set diplstate between two players.
+**************************************************************************/
+void set_diplstate_type(struct player_diplstate *state1,
+                        struct player_diplstate *state2,
+                        enum diplstate_type type)
+{
+  enum diplstate_type max;
+
+  if (type ==  DS_ARMISTICE) {
+    max = DS_PEACE;
+  } else {
+    max = type;
+  }
+
+  max = dst_closest(max, state1->max_state);
+
+  state1->type = type;
+  state2->type = type;
+  state1->max_state = max;
+  state2->max_state = max;
+}
+
+/**********************************************************************//**
+  pplayer clicked the accept button. If they accepted the treaty we check
+  the clauses. If both players have now accepted the treaty we execute the
+  agreed clauses.
 **************************************************************************/
 void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 					int counterpart)
@@ -145,7 +188,10 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
   bool *player_accept, *other_accept;
   enum dipl_reason diplcheck;
   bool worker_refresh_required = FALSE;
-  struct player *pother = valid_player_by_number(counterpart);
+  struct player *pother = player_by_number(counterpart);
+
+  const struct req_context player_ctxt = { .player = pplayer };
+  const struct req_context other_ctxt  = { .player = pother };
 
   if (NULL == pother || pplayer == pother) {
     return;
@@ -167,49 +213,64 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 
   if (!*player_accept) {	/* Tries to accept. */
 
-    /* Check that player who accepts can keep what (s)he promises. */
+    /* Check that player who accepts can keep what they promise. */
 
     clause_list_iterate(ptreaty->clauses, pclause) {
       struct city *pcity = NULL;
 
       if (pclause->from == pplayer) {
-	switch(pclause->type) {
+        struct clause_info *info = clause_info_get(pclause->type);
+
+        if (!are_reqs_active(&player_ctxt, pother,
+                             &(info->giver_reqs), RPT_POSSIBLE)
+            || !are_reqs_active(&other_ctxt, pplayer,
+                                &(info->receiver_reqs), RPT_POSSIBLE)) {
+          log_error("Requirements of a clause between %s and %s not fulfilled",
+                    player_name(pplayer), player_name(pother));
+          return;
+        }
+      }
+
+      if (pclause->from == pplayer || is_pact_clause(pclause->type)) {
+	switch (pclause->type) {
 	case CLAUSE_EMBASSY:
           if (player_has_real_embassy(pother, pplayer)) {
-            freelog(LOG_ERROR, "%s tried to give embassy to %s, who already "
-                    "has an embassy",
-                    player_name(pplayer),
-                    player_name(pother));
+            log_error("%s tried to give embassy to %s, who already "
+                      "has an embassy",
+                      player_name(pplayer), player_name(pother));
             return;
           }
           break;
 	case CLAUSE_ADVANCE:
-          if (!player_invention_reachable(pother, pclause->value)) {
+          if (!research_invention_gettable(research_get(pother),
+                   pclause->value, game.info.tech_trade_allow_holes)) {
 	    /* It is impossible to give a technology to a civilization that
-	     * can never possess it (the client should enforce this). */
-	    freelog(LOG_ERROR, "Treaty: %s can't have tech %s",
-                    nation_rule_name(nation_of_player(pother)),
-		    advance_name_by_player(pplayer, pclause->value));
+	     * can not possess it (the client should enforce this). */
+            log_error("Treaty: %s can't have tech %s",
+                      nation_rule_name(nation_of_player(pother)),
+                      advance_rule_name(advance_by_number(pclause->value)));
             notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
                           _("The %s can't accept %s."),
                           nation_plural_for_player(pother),
-			  advance_name_for_player(pplayer, pclause->value));
+                          advance_name_translation(advance_by_number
+                                                   (pclause->value)));
 	    return;
           }
-	  if (player_invention_state(pplayer, pclause->value) != TECH_KNOWN) {
-	    freelog(LOG_ERROR,
-                    "Nation %s try to give unknown tech %s to nation %s.",
-		    nation_rule_name(nation_of_player(pplayer)),
-		    advance_name_by_player(pplayer, pclause->value),
-		    nation_rule_name(nation_of_player(pother)));
+          if (research_invention_state(research_get(pplayer), pclause->value)
+              != TECH_KNOWN) {
+            log_error("Nation %s try to give unknown tech %s to nation %s.",
+                      nation_rule_name(nation_of_player(pplayer)),
+                      advance_rule_name(advance_by_number(pclause->value)),
+                      nation_rule_name(nation_of_player(pother)));
             notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
 			  _("You don't have tech %s, you can't accept treaty."),
-			  advance_name_for_player(pplayer, pclause->value));
+                          advance_name_translation(advance_by_number
+                                                   (pclause->value)));
 	    return;
 	  }
 	  break;
 	case CLAUSE_CITY:
-	  pcity = game_find_city_by_number(pclause->value);
+	  pcity = game_city_by_number(pclause->value);
 	  if (!pcity) { /* Can't find out cityname any more. */
             notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
 			  _("City you are trying to give no longer exists, "
@@ -244,11 +305,16 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
           break;
 	case CLAUSE_ALLIANCE:
           diplcheck = pplayer_can_make_treaty(pplayer, pother, DS_ALLIANCE);
-          if (diplcheck == DIPL_ALLIANCE_PROBLEM) {
+          if (diplcheck == DIPL_ALLIANCE_PROBLEM_US) {
             notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
                           _("You cannot form an alliance because you are "
                             "at war with an ally of %s."),
                           player_name(pother));
+          } else if (diplcheck == DIPL_ALLIANCE_PROBLEM_THEM) {
+            notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
+                          _("You cannot form an alliance because %s is "
+                            "at war with an ally of yours."),
+                          player_name(pother));
           }
           if (diplcheck != DIPL_OK) {
             return;
@@ -300,14 +366,34 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
                   nclauses);
 
     /* Check that one who accepted treaty earlier still have everything
-       (s)he promised to give. */
+       they promised to give. */
 
     clause_list_iterate(ptreaty->clauses, pclause) {
       struct city *pcity;
+
+      if (pclause->from == pother) {
+        struct clause_info *info = clause_info_get(pclause->type);
+
+        if (!are_reqs_active(&other_ctxt, pplayer,
+                             &(info->giver_reqs), RPT_POSSIBLE)
+            || !are_reqs_active(&player_ctxt, pother,
+                                &(info->receiver_reqs), RPT_POSSIBLE)) {
+          notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
+                        _("Clause requirements are no longer fulfilled. "
+                          "Treaty with %s canceled!"),
+                        nation_plural_for_player(pother));
+          notify_player(pother, NULL, E_DIPLOMACY, ftc_server,
+                        _("Clause requirements are no longer fulfilled. "
+                          "Treaty with %s canceled!"),
+                        nation_plural_for_player(pplayer));
+          return;
+        }
+      }
+
       if (pclause->from == pother) {
 	switch (pclause->type) {
 	case CLAUSE_CITY:
-	  pcity = game_find_city_by_number(pclause->value);
+          pcity = game_city_by_number(pclause->value);
 	  if (!pcity) { /* Can't find out cityname any more. */
             notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
                           _("One of the cities the %s are giving away"
@@ -386,54 +472,84 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
     clause_list_iterate(ptreaty->clauses, pclause) {
       struct player *pgiver = pclause->from;
       struct player *pdest = (pplayer == pgiver) ? pother : pplayer;
-      enum diplstate_type old_diplstate = 
-        pgiver->diplstates[player_index(pdest)].type;
+      struct player_diplstate *ds_giverdest
+        = player_diplstate_get(pgiver, pdest);
+      struct player_diplstate *ds_destgiver
+        = player_diplstate_get(pdest, pgiver);
+      enum diplstate_type old_diplstate = ds_giverdest->type;
+      struct unit_list *pgiver_seen_units BAD_HEURISTIC_INIT(NULL);
+      struct unit_list *pdest_seen_units BAD_HEURISTIC_INIT(NULL);
 
       switch (pclause->type) {
       case CLAUSE_EMBASSY:
         establish_embassy(pdest, pgiver); /* sic */
-        notify_player(pgiver, NULL, E_TREATY_SHARED_VISION, ftc_server,
+        notify_player(pgiver, NULL, E_TREATY_EMBASSY, ftc_server,
                       _("You gave an embassy to %s."),
                       player_name(pdest));
-        notify_player(pdest, NULL, E_TREATY_SHARED_VISION, ftc_server,
+        notify_player(pdest, NULL, E_TREATY_EMBASSY, ftc_server,
                       _("%s allowed you to create an embassy!"),
                       player_name(pgiver));
         break;
       case CLAUSE_ADVANCE:
-        /* It is possible that two players open the diplomacy dialog
-         * and try to give us the same tech at the same time. This
-         * should be handled discreetly instead of giving a core dump. */
-        if (player_invention_state(pdest, pclause->value) == TECH_KNOWN) {
-	  freelog(LOG_VERBOSE,
-                  "Nation %s already know tech %s, that %s want to give them.",
-		  nation_rule_name(nation_of_player(pdest)),
-		  advance_name_by_player(pplayer, pclause->value),
-		  nation_rule_name(nation_of_player(pgiver)));
-          break;
+        {
+          /* It is possible that two players open the diplomacy dialog
+           * and try to give us the same tech at the same time. This
+           * should be handled discreetly instead of giving a core dump. */
+          struct research *presearch = research_get(pdest);
+          const char *advance_name;
+
+          if (research_invention_state(presearch, pclause->value)
+              == TECH_KNOWN) {
+            log_verbose("Nation %s already know tech %s, "
+                        "that %s want to give them.",
+                        nation_rule_name(nation_of_player(pdest)),
+                        advance_rule_name(advance_by_number(pclause->value)),
+                        nation_rule_name(nation_of_player(pgiver)));
+            break;
+          }
+          advance_name =  advance_name_translation(advance_by_number
+                                                   (pclause->value));
+          notify_player(pdest, NULL, E_TECH_GAIN, ftc_server,
+                        _("You are taught the knowledge of %s."),
+                        advance_name);
+
+          if (tech_transfer(pdest, pgiver, pclause->value)) {
+            char research_name[MAX_LEN_NAME * 2];
+
+            research_pretty_name(presearch, research_name,
+                                 sizeof(research_name));
+            notify_research(presearch, pdest, E_TECH_GAIN, ftc_server,
+                            _("You have acquired %s thanks to the %s "
+                              "treaty with the %s."),
+                            advance_name,
+                            nation_adjective_for_player(pdest),
+                            nation_plural_for_player(pgiver));
+            notify_research_embassies
+                (presearch, pgiver, E_TECH_EMBASSY, ftc_server,
+                 /* TRANS: Tech from another player */
+                 Q_("?fromplr:The %s have acquired %s from the %s."),
+                 research_name,
+                 advance_name,
+                 nation_plural_for_player(pgiver));
+
+            script_tech_learned(presearch, pdest,
+                                advance_by_number(pclause->value), "traded");
+            research_apply_penalty(presearch, pclause->value,
+                                   game.server.diplbulbcost);
+            found_new_tech(presearch, pclause->value, FALSE, TRUE);
+          }
         }
-        notify_player(pdest, NULL, E_TECH_GAIN, ftc_server,
-                      _("You are taught the knowledge of %s."),
-                      advance_name_for_player(pdest, pclause->value));
-
-        notify_embassies(pdest, pgiver, NULL, E_TECH_GAIN, ftc_server,
-                         _("The %s have acquired %s from the %s."),
-                         nation_plural_for_player(pdest),
-                         advance_name_for_player(pdest, pclause->value),
-                         nation_plural_for_player(pgiver));
-
-	script_signal_emit("tech_researched", 3,
-			   API_TYPE_TECH_TYPE, advance_by_number(pclause->value),
-			   API_TYPE_PLAYER, pdest,
-			   API_TYPE_STRING, "traded");
-	do_dipl_cost(pdest, pclause->value);
-
-	found_new_tech(pdest, pclause->value, FALSE, TRUE);
-	break;
+        break;
       case CLAUSE_GOLD:
+        {
+          int received = pclause->value
+                         * (100 - game.server.diplgoldcost) / 100;
+          pgiver->economic.gold -= pclause->value;
+          pdest->economic.gold += received;
           notify_player(pdest, NULL, E_DIPLOMACY, ftc_server,
-                        _("You get %d gold."), pclause->value);
-	pgiver->economic.gold -= pclause->value;
-	pdest->economic.gold += pclause->value * (100 - game.info.diplcost) / 100;
+                        PL_("You get %d gold.",
+                            "You get %d gold.", received), received);
+        }
 	break;
       case CLAUSE_MAP:
 	give_map_from_player_to_player(pgiver, pdest);
@@ -455,95 +571,110 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 	break;
       case CLAUSE_CITY:
 	{
-	  struct city *pcity = game_find_city_by_number(pclause->value);
+          struct city *pcity = game_city_by_number(pclause->value);
 
 	  if (!pcity) {
-	    freelog(LOG_ERROR,
-		    "Treaty city id %d not found - skipping clause.",
-		    pclause->value);
+            log_error("Treaty city id %d not found - skipping clause.",
+                      pclause->value);
 	    break;
 	  }
 
           notify_player(pdest, city_tile(pcity), E_CITY_TRANSFER, ftc_server,
-                        _("You receive city of %s from %s."),
+                        _("You receive the city of %s from %s."),
                         city_link(pcity), player_name(pgiver));
 
           notify_player(pgiver, city_tile(pcity), E_CITY_LOST, ftc_server,
-                        _("You give city of %s to %s."),
+                        _("You give the city of %s to %s."),
                         city_link(pcity), player_name(pdest));
 
-	  transfer_city(pdest, pcity, -1, TRUE, TRUE, FALSE);
+          if (transfer_city(pdest, pcity, -1, TRUE, TRUE, FALSE,
+                            !is_barbarian(pdest))) {
+            script_server_signal_emit("city_transferred", pcity, pgiver,
+                                      pdest, "trade");
+          }
 	  break;
 	}
       case CLAUSE_CEASEFIRE:
-	pgiver->diplstates[player_index(pdest)].type=DS_CEASEFIRE;
-	pgiver->diplstates[player_index(pdest)].turns_left = TURNS_LEFT;
-	pdest->diplstates[player_index(pgiver)].type=DS_CEASEFIRE;
-	pdest->diplstates[player_index(pgiver)].turns_left = TURNS_LEFT;
+        if (old_diplstate == DS_ALLIANCE) {
+          pgiver_seen_units = get_units_seen_via_ally(pgiver, pdest);
+          pdest_seen_units = get_units_seen_via_ally(pdest, pgiver);
+        }
+        ds_giverdest->type = DS_CEASEFIRE;
+        ds_giverdest->turns_left = TURNS_LEFT;
+        ds_destgiver->type = DS_CEASEFIRE;
+        ds_destgiver->turns_left = TURNS_LEFT;
         notify_player(pgiver, NULL, E_TREATY_CEASEFIRE, ftc_server,
                       _("You agree on a cease-fire with %s."),
                       player_name(pdest));
         notify_player(pdest, NULL, E_TREATY_CEASEFIRE, ftc_server,
                       _("You agree on a cease-fire with %s."),
                       player_name(pgiver));
-	if (old_diplstate == DS_ALLIANCE) {
-	  update_players_after_alliance_breakup(pgiver, pdest);
-	}
+        if (old_diplstate == DS_ALLIANCE) {
+          update_players_after_alliance_breakup(pgiver, pdest,
+                                                pgiver_seen_units,
+                                                pdest_seen_units);
+          unit_list_destroy(pgiver_seen_units);
+          unit_list_destroy(pdest_seen_units);
+        }
 
         worker_refresh_required = TRUE;
 	break;
       case CLAUSE_PEACE:
-	pgiver->diplstates[player_index(pdest)].type = DS_ARMISTICE;
-	pdest->diplstates[player_index(pgiver)].type = DS_ARMISTICE;
-	pgiver->diplstates[player_index(pdest)].turns_left = TURNS_LEFT;
-	pdest->diplstates[player_index(pgiver)].turns_left = TURNS_LEFT;
-	pgiver->diplstates[player_index(pdest)].max_state = 
-          MAX(DS_PEACE, pgiver->diplstates[player_index(pdest)].max_state);
-	pdest->diplstates[player_index(pgiver)].max_state = 
-          MAX(DS_PEACE, pdest->diplstates[player_index(pgiver)].max_state);
+        if (old_diplstate == DS_ALLIANCE) {
+          pgiver_seen_units = get_units_seen_via_ally(pgiver, pdest);
+          pdest_seen_units = get_units_seen_via_ally(pdest, pgiver);
+        }
+        set_diplstate_type(ds_giverdest, ds_destgiver, DS_ARMISTICE);
+        ds_giverdest->turns_left = TURNS_LEFT;
+        ds_destgiver->turns_left = TURNS_LEFT;
         notify_player(pgiver, NULL, E_TREATY_PEACE, ftc_server,
-                      /* TRANS: ... the Poles ... Polish territory. */
+                      /* TRANS: ... the Poles ... Polish territory */
                       PL_("You agree on an armistice with the %s. In %d turn, "
                           "it will become a peace treaty. Move your "
-                          "units out of %s territory.",
+                          "military units out of %s territory to avoid them "
+                          "being disbanded.",
                           "You agree on an armistice with the %s. In %d turns, "
-                          "it will become a peace treaty. Move your "
-                          "units out of %s territory.",
+                          "it will become a peace treaty. Move any "
+                          "military units out of %s territory to avoid them "
+                          "being disbanded.",
                           TURNS_LEFT),
                       nation_plural_for_player(pdest),
                       TURNS_LEFT,
                       nation_adjective_for_player(pdest));
         notify_player(pdest, NULL, E_TREATY_PEACE, ftc_server,
-                      /* TRANS: ... the Poles ... Polish territory. */
+                      /* TRANS: ... the Poles ... Polish territory */
                       PL_("You agree on an armistice with the %s. In %d turn, "
                           "it will become a peace treaty. Move your "
-                          "units out of %s territory.",
+                          "military units out of %s territory to avoid them "
+                          "being disbanded.",
                           "You agree on an armistice with the %s. In %d turns, "
-                          "it will become a peace treaty. Move your "
-                          "units out of %s territory.",
+                          "it will become a peace treaty. Move any "
+                          "military units out of %s territory to avoid them "
+                          "being disbanded.",
                           TURNS_LEFT),
                       nation_plural_for_player(pgiver),
                       TURNS_LEFT,
                       nation_adjective_for_player(pgiver));
-	if (old_diplstate == DS_ALLIANCE) {
-	  update_players_after_alliance_breakup(pgiver, pdest);
-	}
+        if (old_diplstate == DS_ALLIANCE) {
+          update_players_after_alliance_breakup(pgiver, pdest,
+                                                pgiver_seen_units,
+                                                pdest_seen_units);
+          unit_list_destroy(pgiver_seen_units);
+          unit_list_destroy(pdest_seen_units);
+        }
 
         worker_refresh_required = TRUE;
 	break;
       case CLAUSE_ALLIANCE:
-	pgiver->diplstates[player_index(pdest)].type=DS_ALLIANCE;
-	pdest->diplstates[player_index(pgiver)].type=DS_ALLIANCE;
-	pgiver->diplstates[player_index(pdest)].max_state = 
-          MAX(DS_ALLIANCE, pgiver->diplstates[player_index(pdest)].max_state);
-	pdest->diplstates[player_index(pgiver)].max_state = 
-          MAX(DS_ALLIANCE, pdest->diplstates[player_index(pgiver)].max_state);
+        set_diplstate_type(ds_giverdest, ds_destgiver, DS_ALLIANCE);
         notify_player(pgiver, NULL, E_TREATY_ALLIANCE, ftc_server,
                       _("You agree on an alliance with %s."),
                       player_name(pdest));
         notify_player(pdest, NULL, E_TREATY_ALLIANCE, ftc_server,
                       _("You agree on an alliance with %s."),
                       player_name(pgiver));
+        give_allied_visibility(pgiver, pdest);
+        give_allied_visibility(pdest, pgiver);
 
         worker_refresh_required = TRUE;
 	break;
@@ -560,8 +691,8 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
          * within radius of our own city. */
         worker_refresh_required = TRUE;
 	break;
-      case CLAUSE_LAST:
-        freelog(LOG_ERROR, "Received bad clause type");
+      case CLAUSE_COUNT:
+        fc_assert(pclause->type != CLAUSE_COUNT);
         break;
       }
 
@@ -579,36 +710,46 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
     }
 
   cleanup:
-    treaty_list_unlink(treaties, ptreaty);
+    treaty_list_remove(treaties, ptreaty);
     clear_treaty(ptreaty);
     free(ptreaty);
-    send_player_info(pplayer, NULL);
-    send_player_info(pother, NULL);
+    send_player_all_c(pplayer, NULL);
+    send_player_all_c(pother, NULL);
   }
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Create an embassy. pplayer gets an embassy with aplayer.
-****************************************************************************/
+**************************************************************************/
 void establish_embassy(struct player *pplayer, struct player *aplayer)
 {
   /* Establish the embassy. */
   BV_SET(pplayer->real_embassy, player_index(aplayer));
-  send_player_info(pplayer, pplayer);
-  send_player_info(pplayer, aplayer);  /* update player dialog with embassy */
-  send_player_info(aplayer, pplayer);  /* INFO_EMBASSY level info */
+
+  player_list_iterate(team_members(pplayer->team), teammate) {
+    /* Knowledge that pplayer has an embassy now */
+    send_player_all_c(pplayer, teammate->connections);
+    /* INFO_EMBASSY level info */
+    send_player_all_c(aplayer, teammate->connections);
+  } player_list_iterate_end;
+
+  /* update player dialog with embassy */
+  send_player_all_c(pplayer, aplayer->connections);
+
+  /* Send research info */
+  send_research_info(research_get(aplayer), pplayer->connections);
 }
 
-/**************************************************************************
-...
+/**********************************************************************//**
+  Handle request to remove clause from treaty.
 **************************************************************************/
 void handle_diplomacy_remove_clause_req(struct player *pplayer,
 					int counterpart, int giver,
 					enum clause_type type, int value)
 {
   struct Treaty *ptreaty;
-  struct player *pgiver = valid_player_by_number(giver);
-  struct player *pother = valid_player_by_number(counterpart);
+  struct player *pgiver = player_by_number(giver);
+  struct player *pother = player_by_number(counterpart);
 
   if (NULL == pother || pplayer == pother || NULL == pgiver) {
     return;
@@ -632,16 +773,16 @@ void handle_diplomacy_remove_clause_req(struct player *pplayer,
   }
 }
 
-/**************************************************************************
-...
+/**********************************************************************//**
+  Handle request to add clause to treaty between two players.
 **************************************************************************/
 void handle_diplomacy_create_clause_req(struct player *pplayer,
 					int counterpart, int giver,
 					enum clause_type type, int value)
 {
   struct Treaty *ptreaty;
-  struct player *pgiver = valid_player_by_number(giver);
-  struct player *pother = valid_player_by_number(counterpart);
+  struct player *pgiver = player_by_number(giver);
+  struct player *pother = player_by_number(counterpart);
 
   if (NULL == pother || pplayer == pother || NULL == pgiver) {
     return;
@@ -653,17 +794,18 @@ void handle_diplomacy_create_clause_req(struct player *pplayer,
 
   ptreaty = find_treaty(pplayer, pother);
 
-  if (ptreaty && add_clause(ptreaty, pgiver, type, value)) {
-    /* 
-     * If we are trading cities, then it is possible that the
-     * dest is unaware of it's existence.  We have 2 choices,
-     * forbid it, or lighten that area.  If we assume that
+  if (ptreaty != NULL
+      && add_clause(ptreaty, pgiver, type, value, NULL)) {
+    /*
+     * If we are trading cities, then it is possible that
+     * the dest is unaware of it's existence. We have 2 choices,
+     * forbid it, or lighten that area. If we assume that
      * the giver knows what they are doing, then 2. is the
      * most powerful option - I'll choose that for now.
      *                           - Kris Bubendorfer
      */
     if (type == CLAUSE_CITY) {
-      struct city *pcity = game_find_city_by_number(value);
+      struct city *pcity = game_city_by_number(value);
 
       if (pcity && !map_is_known_and_seen(pcity->tile, pother, V_MAIN))
 	give_citymap_from_player_to_player(pcity, pplayer, pother);
@@ -680,8 +822,9 @@ void handle_diplomacy_create_clause_req(struct player *pplayer,
   }
 }
 
-/**************************************************************************
-...
+/**********************************************************************//**
+  Cancel meeting. No sanity checking of input parameters, so don't call
+  this with input directly from untrusted source.
 **************************************************************************/
 static void really_diplomacy_cancel_meeting(struct player *pplayer,
 					    struct player *pother)
@@ -702,19 +845,19 @@ static void really_diplomacy_cancel_meeting(struct player *pplayer,
     notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
                   _("Meeting with %s canceled."), 
                   player_name(pother));
-    treaty_list_unlink(treaties, ptreaty);
+    treaty_list_remove(treaties, ptreaty);
     clear_treaty(ptreaty);
     free(ptreaty);
   }
 }
 
-/**************************************************************************
-...
+/**********************************************************************//**
+  Handle meeting cancelling request.
 **************************************************************************/
 void handle_diplomacy_cancel_meeting_req(struct player *pplayer,
 					 int counterpart)
 {
-  struct player *pother = valid_player_by_number(counterpart);
+  struct player *pother = player_by_number(counterpart);
 
   if (NULL == pother || pplayer == pother) {
     return;
@@ -723,13 +866,13 @@ void handle_diplomacy_cancel_meeting_req(struct player *pplayer,
   really_diplomacy_cancel_meeting(pplayer, pother);
 }
 
-/**************************************************************************
-...
+/**********************************************************************//**
+  Handle meeting opening request.
 **************************************************************************/
 void handle_diplomacy_init_meeting_req(struct player *pplayer,
 				       int counterpart)
 {
-  struct player *pother = valid_player_by_number(counterpart);
+  struct player *pother = player_by_number(counterpart);
 
   if (NULL == pother || pplayer == pother) {
     return;
@@ -739,8 +882,8 @@ void handle_diplomacy_init_meeting_req(struct player *pplayer,
     return;
   }
 
-  if (get_player_bonus(pplayer, EFT_NO_DIPLOMACY)
-      || get_player_bonus(pother, EFT_NO_DIPLOMACY)) {
+  if (get_player_bonus(pplayer, EFT_NO_DIPLOMACY) > 0
+      || get_player_bonus(pother, EFT_NO_DIPLOMACY) > 0) {
     notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
                   _("Your diplomatic envoy was decapitated!"));
     return;
@@ -762,7 +905,7 @@ void handle_diplomacy_init_meeting_req(struct player *pplayer,
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Send information on any on-going diplomatic meetings for connection's
   player.  For re-connections.
 **************************************************************************/
@@ -777,7 +920,7 @@ void send_diplomatic_meetings(struct connection *dest)
     struct Treaty *ptreaty = find_treaty(pplayer, other);
 
     if (ptreaty) {
-      assert(pplayer != other);
+      fc_assert_action(pplayer != other, continue);
       dsend_packet_diplomacy_init_meeting(dest, player_number(other),
                                           player_number(pplayer));
       clause_list_iterate(ptreaty->clauses, pclause) {
@@ -787,21 +930,22 @@ void send_diplomatic_meetings(struct connection *dest)
                                              pclause->type,
                                              pclause->value);
       } clause_list_iterate_end;
+
       if (ptreaty->plr0 == pplayer) {
         dsend_packet_diplomacy_accept_treaty(dest, player_number(other),
-                                             ptreaty->accept0, 
+                                             ptreaty->accept0,
                                              ptreaty->accept1);
       } else {
         dsend_packet_diplomacy_accept_treaty(dest, player_number(other),
-                                             ptreaty->accept1, 
+                                             ptreaty->accept1,
                                              ptreaty->accept0);
       }
     }
   } players_iterate_end;
 }
 
-/**************************************************************************
-...
+/**********************************************************************//**
+  Cancels all meetings of player.
 **************************************************************************/
 void cancel_all_meetings(struct player *pplayer)
 {
@@ -812,7 +956,7 @@ void cancel_all_meetings(struct player *pplayer)
   } players_iterate_end;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Reject all treaties currently being negotiated
 **************************************************************************/
 void reject_all_treaties(struct player *pplayer)
@@ -836,3 +980,10 @@ void reject_all_treaties(struct player *pplayer)
   } players_iterate_end;
 }
 
+/**********************************************************************//**
+  Get treaty list
+**************************************************************************/
+struct treaty_list *get_all_treaties(void)
+{
+  return treaties;
+}
